# Transfer Function to State Space Conversion Comparison

This report compares the poles of state-space realizations obtained using different implementations:

1. Our implementation
2. SLYCOT implementation
3. MATLAB implementation

### Comparison Table of Poles

| System | Our Implementation | SLYCOT Implementation | MATLAB Implementation |
|--------|-------------------|----------------------|----------------------|
| $  \frac{1s + 1}{s^2 + 3s + 2}  $ | -2.0 + 0.0j | -2.0 + 0.0j | -2.0 + 0.0j<br>-1.0 + 0.0j |
| $ \begin{bmatrix} \frac{0s^2 + 0s + 1}{s^2 + 0.4s + 3} & \frac{s^2 + 0s + 0}{s^2 + s + 0} \\ \frac{3s^2 - 1s + 1}{s^2 + 0.4s + 3} & \frac{0s^2 + s + 0}{s^2 + s + 0} \\ \frac{0s^2 + 0s + 1}{s^2 + 0.4s + 3} & \frac{0s^2 + 2s + 0}{s^2 + s + 0} \end{bmatrix} $ | 0.0 +0.0j<br>-0.2+1.7205j<br>-0.2-1.7205j<br>-1.0 +0.0j | -0.2+1.7205j<br>-0.2-1.7205j<br>-0.2+1.7205j<br>-0.2-1.7205j<br>-0.2+1.7205j<br>-0.2-1.7205j<br>-1. +0.j<br>-1. +0.j<br>-1. +0.j  | -0.2 + 1.7205j<br>-0.2 - 1.7205j<br>0.0 + 0.0j<br>-1.0 + 0.0j |
| $ \begin{bmatrix} \frac{s^3 + 6s^2 + 12s + 7}{s^3 + 6s^2 + 11s + 6} & \frac{0s^3 + s^2 + 4s + 3}{s^3 + 6s^2 + 11s + 6} \\ \frac{0s^3 + 0s^2 + s + 1}{s^3 + 6s^2 + 11s + 6} & \frac{s^3 + 8s^2 + 20s + 15}{s^3 + 6s^2 + 11s + 6} \end{bmatrix} $ | -3.0 + 0.0j<br>-2.0 + 0.0j<br>-1.0 + 0.0j | -3.0 + 0.0j<br>-2.0 + 0.0j<br>-1.0 + 0.0j<br>-2.0 + 0.0j<br>-2.0 + 0.0j<br>-3.0 + 0.0j<br>-2.0 + 0.0j | -3.0 + 0.0j<br>-2.0 + 0.0j<br>-1.0 + 0.0j<br>-3.0 + 0.0j<br>-2.0 + 0.0j<br>-1.0 + 0.0j |

### Comparison Table of Transmission Zeros

| System | Our Implementation | SLYCOT Implementation | MATLAB Implementation |
|--------|-------------------|----------------------|----------------------|
| $  \frac{1s + 1}{s^2 + 3s + 2}  $ | [] | [] | -1.0 + 0.0j |
| $ \begin{bmatrix} \frac{0s^2 + 0s + 1}{s^2 + 0.4s + 3} & \frac{s^2 + 0s + 0}{s^2 + s + 0} \\ \frac{3s^2 - 1s + 1}{s^2 + 0.4s + 3} & \frac{0s^2 + s + 0}{s^2 + s + 0} \\ \frac{0s^2 + 0s + 1}{s^2 + 0.4s + 3} & \frac{0s^2 + 2s + 0}{s^2 + s + 0} \end{bmatrix} $ | [] | -2.87e-17 + 0.0j<br>0.167 + 0.553j<br>0.167 - 0.553j | -3.1991e-17 + 0.0j |
| $ \begin{bmatrix} \frac{s^3 + 6s^2 + 12s + 7}{s^3 + 6s^2 + 11s + 6} & \frac{0s^3 + s^2 + 4s + 3}{s^3 + 6s^2 + 11s + 6} \\ \frac{0s^3 + 0s^2 + s + 1}{s^3 + 6s^2 + 11s + 6} & \frac{s^3 + 8s^2 + 20s + 15}{s^3 + 6s^2 + 11s + 6} \end{bmatrix} $ | -1.0 + 0.0j<br>-0.333 + 0.0j | -3.618 + 0.0j<br>-2.5 - 0.866j<br>-2.5 + 0.866j<br>-1.382 + 0.0j | -3.2328 + 0.7926j<br>-3.2328 - 0.7926j<br>-1.5344 + 0.0000j<br>-2.0000 + 0.0000j<br>-3.0000 + 0.0000j<br>-1.0000 + 0.0000j |

### Speed Comparison: Our Implementation vs SLYCOT

The following table presents a benchmark of execution times (in seconds) for our `tf2ss` implementation versus the SLYCOT-based implementation, using random stable MIMO systems of various sizes with system order 4. Each entry reports the minimum time over three trials for both methods. The random systems are generated by placing poles in the left half-plane to ensure stability, and numerators are randomly generated and padded to match the denominator order.

| Outputs | Inputs | tf2ss [s]   | ctrl_tf2ss [s] | Slowdown |
|---------|--------|-------------|---------------|----------|
| 1       | 1      | 0.000751    | 0.000238      | 3.15     |
| 2       | 2      | 0.005714    | 0.001017      | 5.62     |
| 3       | 3      | 0.034349    | 0.002559      | 13.42    |
| 4       | 4      | 0.162059    | 0.005334      | 30.38    |
| 5       | 5      | 0.533696    | 0.010380      | 51.41    |
| 6       | 6      | 1.438460    | 0.018480      | 77.84    |
| 8       | 8      | 7.557856    | 0.052440      | 144.12   |
| 10      | 10     | 28.021714   | 0.109455      | 256.01   |
